--------------------------------------------------------------------------------
-- Copyright  2019								|
-- All Rights Reserved.								|
-- 										|
-- Permission to use, copy, modify, and distribute this software in any way	|
-- is not granted without expressed written agreement. This copyright applies	|
-- to the all code, data and artwork related to this project and all associated	|
-- projects published by the author.						|
--										|
-- For more information on copyright is available from sources on-line.		|
-- Use an actual factual one!							|
--------------------------------------------------------------------------------

local AddonName, NS = ...

local L
local AddonList, Container

local Extra, Offset = 25, 25

local Selected = { addon="", reset="" }

local function ConfigureAddons(parent)
	local OpenOptions, LastIcon
	L = NS.TEXTS
	if Container then
		Container:SetShown(not Container:IsShown())
		return Container
	end

	local function CopySettings(character, charsettings)
-- Insert stuff here!
	end

	local function GetReset(self)
		local parent = self:GetParent()
		local addon = parent.Table[self.index].addon
		local assignOnly = parent.AssignOnly
		local isAddon, isAssign
		if parent.LastSelection and parent.LastSelection == addon then
			isAddon = true
		end
		if assignOnly == self.index then
			isAssign = true
		end
		return isAddon, isAssign
	end
	
	local function SetPortrait(self, checked, disabled)
		if disabled then
			self:SetNormalTexture(Container.DisabledTexture:GetTexture())
			return
		end
		if checked then
			local t = self:GetNormalTexture()
			SetPortraitTexture(t, "player")
		else
			self:SetNormalTexture(Container.ResetTexture:GetTexture())
		end
	end
	
	local function UpdateReset(self, table, index, data, savedvalue)
		local parent = self:GetParent()
		local addon = parent.Table[self.index].addon
		local isAddon, isAssign = GetReset(self)
		if isAddon then
			parent.LastSelection = nil
			parent.LastSelectionIndex = nil
			Container.ResetThisAddon:Hide()
			if isAssign then
				self.Icon:GetScript("OnClick")(self.Icon)
			end
		else
			if parent.AssignOnly ~= self.index then
				parent.AssignOnly = nil
			end
			parent.LastSelection = addon
			parent.LastSelectionIndex = self.index
			Container.ResetThisAddon:SetTooltip(L.Reset .. " " .. parent.LastSelection)
			Container.ResetThisAddon:Show()
		end
	end

	local function ResetButtonUpdate(self, table, index, data, savedvalue)
		local T = self:GetNormalTexture()
		local Inactive
		if not table[index].nosettings then
--			Inactive = NS:GetSettings("Addons")[self:GetParent().Table[self.index].addon].inactive
			Inactive = NS:GetSettings("Addons")[table[self.index].addon].inactive
		end
		if Inactive then
			T:SetVertexColor(1, 1, 1)
			SetPortrait(self.Icon, nil, true)
			self.Icon:Disable()
			self:Disable()
			return
		end
		self:Enable()
		if not table[index].assign then
			self.Icon:Disable()
		else
			self.Icon:Enable()
		end
		local isAddon, isAssign = GetReset(self)
		if isAddon then
			T:SetVertexColor(1, 0, 0)
		else
			T:SetVertexColor(1, 1, 1)
		end
		SetPortrait(self.Icon, isAssign)
	end 
	
	local function ResetIconTooltip(self)
		local parent = self:GetParent()
		local list = parent:GetParent()
		GameTooltip:SetOwner(self, "ANCHOR_TOP", 0, 4)
		if list.AssignOnly == parent.index then
--			GameTooltip:SetText(L.AddonConfig)
			GameTooltip:SetText(L.AddonConfig, nil, nil, nil, nil, true)
		else
--			GameTooltip:SetText(L.CharacterConfig)
			GameTooltip:SetText(L.CharacterConfig, nil, nil, nil, nil, true)
		end
		GameTooltip:Show()
	end
	
	local function ResetButtonCreate(self) -- self = button base
		NS:SetScrollHighlight(self)
		self.Icon = CreateFrame("Button", nil, self)
		self.Icon:SetPoint("TOPLEFT", 2, -3)
		self.Icon:SetPoint("BOTTOMRIGHT", self, "BOTTOMLEFT", 26, 3)
		self.Icon:SetNormalTexture(Container.ResetTexture:GetTexture())
		self.Icon:SetHighlightTexture(NS.TexturePath.."OptionButton")
		self.Icon:SetScript("OnClick", function(self)
			GameTooltip:Hide()
			local parent = self:GetParent()
			local list = parent:GetParent()
			if list.AssignOnly == parent.index then
				list.AssignOnly = nil
				SetPortrait(self)
			else
				list.AssignOnly = parent.index
				SetPortrait(self, true)
			end
			list:InitFunc()
			ResetIconTooltip(self)
		end)
		self.Text = NS.Lib:CreateFont("Text", self)
		self.Text:SetPoint("LEFT", self.Icon, "RIGHT", 4, 0)
		self:HookScript("OnClick", function(self)
			UpdateReset(self)
		end)
		self.Icon:SetScript("OnEnter",  ResetIconTooltip)
		self.Icon:SetScript("OnLeave",  function() GameTooltip:Hide() end)
	end
	
	local function UpdateActive(self, clicked)
		if Container.LastOpen then
			Container.LastOpen:Hide()
			Container.LastOpen = nil
			if LastIcon then
				LastIcon:SetNormalTexture(NS.TexturePath.."AddonOptionsOpen")
				LastIcon = nil
			end
		end
		local Entry = self:GetParent().Table[self.index]
		local Settings = NS:GetSettings("Addons")[Entry.addon]
		if not Settings then
			L.print(L.ErrorNoAddonSettings:format(Entry.addon))
			return
		end
		if Entry.reloadonchange then
			if Entry.ActionReload == nil then
				Entry.ActionReload = Settings.inactive or false
			end
		end
		if Settings.inactive then
			self.Folder:SetTexture(NS.TexturePath.."AddonDisabled")
		else
			self.Folder:SetTexture(NS.TexturePath.."AddOnEnabled")
		end
		local width = 0
		if clicked then
			if not OpenOptions or OpenOptions ~= self.index then
				OpenOptions = self.index
				if NS:GetAddonOptions(Entry.addon) then
					Container.LastOpen = NS:GetAddonOptions(Entry.addon)(Container, Entry)
					if not Container.LastOpen.Docked then
						Container.LastOpen.Docked = true
						Container.LastOpen:ClearAllPoints()
						Container.LastOpen:SetPoint("TOPLEFT", Container, "TOPRIGHT", 0, 0)
					end
					width = Container.LastOpen:GetWidth()
				end
			else
				OpenOptions = nil
			end
		end
		if Settings.configs and not Settings.inactive then
			self.Configurable = true
			self.Options:Show()
			if OpenOptions and OpenOptions == self.index then
				self.Options:SetNormalTexture(NS.TexturePath.."AddonOptionsClose")
				LastIcon = self.Options
			end
		else
			self.Configurable = false
			self.Options:Hide()
		end	
		if width == 0 then
			if OpenOptions and OpenOptions == self.index then
				OpenOptions = nil
				self.Options:SetNormalTexture(NS.TexturePath.."AddonOptionsOpen")
			end
		end
	end

	local function AddonButtonCreate(self) -- self = button base
		NS:SetScrollHighlight(self)

		self.Folder = self:CreateTexture(nil, "BACKGROUND", nil, 1)
		self.Folder:SetPoint("TOPLEFT", 3, -3)
		self.Folder:SetPoint("BOTTOMRIGHT", self, "BOTTOMLEFT", 26, 3)
		self.Folder:SetTexture(NS.TexturePath.."AddOnEnabled")
		self.Options = CreateFrame("BUTTON", nil, self)
		self.Options:SetHighlightTexture(NS.Lib.TexturePath.."DropButtonHighlight", "ADD")
		self.Options:SetPoint("TOPRIGHT", -2, -2)
		self.Options:SetPoint("BOTTOMLEFT", self, "BOTTOMRIGHT", -24, 2)
		self.Options:SetNormalTexture(NS.TexturePath.."AddonOptionsOpen")
		self.Options:Hide()
		self.Options:SetScript("OnClick", function(self)
			UpdateActive(self:GetParent(), true)
			PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON)
		end)
		self.Text = NS.Lib:CreateFont("Text", self)
		self.Text:SetPoint("TOPLEFT", self.Folder, "TOPRIGHT", 4, 0)
		self.Text:SetPoint("BOTTOMLEFT", self.Folder, "BOTTOMRIGHT", 4, 0)
		self.Text:SetPoint("RIGHT", self.Options, "LEFT", -2, 0)
		self:HookScript("OnClick", function(self)
			local Entry = self:GetParent().Table[self.index]
			local Settings = NS:GetSettings("Addons")[Entry.addon]
			Settings.inactive = not Settings.inactive
			UpdateActive(self)
			if Entry.reloadonchange and type(Entry.reloadonchange) == "function" then
				Entry.reloadonchange(Settings.inactive)
			end
			local InActive = Settings.inactive or false
			FauxMazzleOptions:SetReloadHighlight(Entry.ActionReload ~= InActive)
			Container.ResetList:InitFunc()
		end)
	end
	
	local function AddonButtonUpdate(self, table, index, data, savedvalue)
--		if not Container.AddonList then return end
		UpdateActive(self)
	end
	
	Container = CreateFrame("Frame", "$parentAddOns", parent)
	Container.ResetTexture = Container:CreateTexture()
	Container.ResetTexture:SetTexture(NS.TexturePath.."Settings")
	Container.DisabledTexture = Container:CreateTexture()
	Container.DisabledTexture:SetTexture(NS.TexturePath.."Cancel")

	AddonList = {}
	ConfigList = {}
	local ConfigFuncs = NS:GetConfigFuncs()
	for k, v in pairs(ConfigFuncs.addons) do	
		if not v.exclude then
			tinsert(AddonList, { display=v.display, addon=k, reloadonchange=v.reloadonchange })
			if ConfigFuncs[v.order] and ConfigFuncs[v.order][v.index].config then
				tinsert(ConfigList, { display=v.display, addon=k, config=ConfigFuncs[v.order][v.index].config, assign=ConfigFuncs[v.order][v.index].assign })
			end
		end
	end
	tinsert(ConfigList, { display=L.ChatGroups, addon="ChatGroups", nosettings=true, config= NS.DefaultsInitialiseChat })
	sort(AddonList, function(a, b)
		return(a.display < b.display)
	end)
	sort(ConfigList, function(a, b)
		return(a.display < b.display)
	end)
	Container.AddonList = NS.Lib:CreateScrollList("AddonList", Container, Selected, "addon", AddonList, "display", "addon", 8, AddonButtonUpdate, AddonButtonCreate)
	Container.AddonList:SetSize(140, 8 * 30)
	Container.AddonList:SetPoint("TOPLEFT", 0, -Offset)
	Container:SetSize(Container.AddonList:GetWidth() + Extra, Container.AddonList:GetHeight() + Offset + 5)
	Container:SetScript("OnHide", function(self)
		if self.LastOpen then
			OpenOptions = nil
			for i=1, #self.AddonList.Buttons do
				UpdateActive(self.AddonList.Buttons[i])
			end
		end
		self.ResetAddons:SetChecked(false)
		self.ResetAddons:GetScript("OnClick")(self.ResetAddons)
	end)

	Container.ResetAddons = NS.Lib:CreateCheckButton("Reconfigure", Container, Container, "ResetShown")
	Container.ResetAddons:SetPoint("TOPLEFT", 5, -2)
	NS.Lib:SetLabel(Container.ResetAddons, L.Defaults, "RIGHT", 0, 0)
	Container.ResetAddons.PostUpdate = function(self)
		Container.AddonList:SetShown(not self:GetChecked())
		Container.ResetList:SetShown(self:GetChecked())
		Container.ResetThisAddon:Hide()
		Selected.reset = ""
	end
	Container.ResetList = NS.Lib:CreateScrollList("ResetList", Container, Selected, "reset", ConfigList, "display", "addon", 8, ResetButtonUpdate, ResetButtonCreate)
	Container.ResetList:SetSize(Container.AddonList:GetSize())
	Container.ResetList:SetPoint(Container.AddonList:GetPoint(1))
	Container.ResetList:Hide()

	local ButtonSettings = { id="ResetThisAddon", nolocale=true, text="", tip=L.Reset, icon="Settings"}
	Container.ResetThisAddon = NS:CreateMenuButton("ResetThisAddon", Container, "Button", ButtonSettings, 20, true)
	Container.ResetThisAddon:Hide()
	Container.ResetThisAddon:SetSize(80, 20)
	Container.ResetThisAddon.Background:SetVertexColor(1, 0, 0)
	Container.ResetThisAddon.Background:SetAlpha(0.7)
	Container.ResetThisAddon:SetPoint("TOPRIGHT", 0, -2)
	Container.ResetThisAddon:SetText(L.Reset)
	Container.ResetThisAddon:HookScript("OnClick", function(self)
		if Container.ResetList.LastSelection then
			local Settings = Container.ResetList.Table[Container.ResetList.LastSelectionIndex]
			if Container.ResetList.AssignOnly and Container.ResetList.AssignOnly == Container.ResetList.LastSelectionIndex then
				if Settings.assign then
					Settings.assign(true)
				end
			else
				if Settings.config then
					Settings.config(true)
				end
				if Settings.assign then
					Settings.assign(true)
				end
			end
		end
		FauxMazzleOptions:SetReloadHighlight(true)
		self:Hide()
	end)
	Container.ResetThisAddon:HookScript("OnHide", function(self)
		Container.ResetList.LastSelection = nil
		Container.ResetList.LastSelectionIndex = nil
		Container.ResetList.AssignOnly = nil
		Container.ResetList:InitFunc()
	end)

	return Container
end

NS:RegisterModule("Addons", "Addons", "Addons", ConfigureAddons)
